#include <Canbus.h>
#include <defaults.h>
#include <global.h>
#include <mcp2515.h>
#include <mcp2515_defs.h>
#include <Wire.h>
#include <TimerOne.h>			// TimerOne library

#define CANSPEED_125_16M	7	// CAN speed at 125 kbps@16M osc  (CNF1 BRP, 16/Fosc)
#define CANSPEED_250_16M	3	// CAN speed at 250 kbps@16M osc  (CNF1 BRP, 8/Fosc)
#define CANSPEED_500_16M	1	// CAN speed at 500 kbps@16M osc  (CNF1 BRP, 4/Fosc)
//
#define CANSPEED_125_8M		3	// CAN speed at 125 kbps@8M osc   (CNF1 BRP, 8/Fosc)
#define CANSPEED_250_8M		1	// CAN speed at 250 kbps@8M osc   (CNF1 BRP, 4/Fosc)
#define CANSPEED_500_8M		0	// CAN speed at 500 kbps@8M osc   (CNF1 BRP, 2/Fosc)


///////////////////////////////////
// timer interrupt vector
///////////////////////////////////
volatile int	time_flag;
volatile int	cmd_flag;
volatile int	reset_flag;

volatile int	gz0, gz1;
volatile int	at0, at1, at2, at3;
volatile int	cmd;
volatile int	rx_ptr;
char			rx_str[256];
char			tx_str[256];

void timer1_func()	// 100ms
{
	time_flag	= 1;
}


void setup()
{
//	Canbus.init(CANSPEED_250_16M);		// can bus shield (OSC 16MHz)
	Canbus.init(CANSPEED_250_8M);		// can bus board (OSC 8MHz)

	cmd_flag	= 0;
	reset_flag	= 0;
	rx_ptr		= 0;
	Serial.begin(115200);

	time_flag	= 0;
	Timer1.initialize(100000);			// unit: us
	Timer1.attachInterrupt(timer1_func);
}


void loop()
{
	tCAN message;

	// receive CAN message
	if (mcp2515_check_message()) {
		if (mcp2515_get_message(&message)) {
			if (message.id == 0x102) {
				gz0	= message.data[0];
				gz1	= message.data[1];
				at0	= message.data[4];
				at1	= message.data[5];
				at2	= message.data[6];
				at3	= message.data[7];
			}
		}
	}

	// send gimbal command (at 100ms boundary)
	if (time_flag == 1) {
		time_flag	= 0;

		if (cmd_flag == 1) {
			cmd_flag	= 0;

			message.id				= 0x100;
			message.header.rtr		= 0;
			message.header.length	= 1;
			message.data[0]			= cmd;
			message.data[1]			= 0;
			message.data[2]			= 0;
			message.data[3]			= 0;
			message.data[4]			= 0;
			message.data[5]			= 0;
			message.data[6]			= 0;
			message.data[7]			= 0;
			mcp2515_bit_modify(CANCTRL, (1<<REQOP2)|(1<<REQOP1)|(1<<REQOP0), 0);
			mcp2515_send_message(&message);

			sprintf(tx_str, "%02x%02x%02x%02x%02x%02x\x0D\x0A", gz0, gz1, at0, at1, at2, at3);
			Serial.print(tx_str);	// return
		}
	}

	if (reset_flag == 1) {
		reset_flag	= 0;

		message.id				= 0x101;
		message.header.rtr		= 0;
		message.header.length	= 0;
		message.data[0]			= 0;
		message.data[1]			= 0;
		message.data[2]			= 0;
		message.data[3]			= 0;
		message.data[4]			= 0;
		message.data[5]			= 0;
		message.data[6]			= 0;
		message.data[7]			= 0;
		mcp2515_bit_modify(CANCTRL, (1<<REQOP2)|(1<<REQOP1)|(1<<REQOP0), 0);
		mcp2515_send_message(&message);

		sprintf(tx_str, "+\x0D\x0A");
		Serial.print(tx_str);		// return
	}

	// check serial
	if (Serial.available() > 0) {
		char	tmp;
		int		ch;
		ch	= Serial.read();
		if (ch >= 0) {
			rx_str[rx_ptr]		= ch;
			rx_str[rx_ptr + 1]	= '\0';
			if (rx_ptr < 250) {
				rx_ptr++;
			}

			if (ch == 0x0A) {
				if (rx_str[0] == '*') {
					rx_ptr		= 0;
					reset_flag	= 1;
				}
				else {
					sscanf(rx_str, "%d\n", &cmd);
					cmd_flag  = 1;
				}
				rx_ptr = 0;
			} // ch == 0x0A
		} // ch >= 0
	} // if
}
// end of file
